Needs["CCompilerDriver`"];

prefix = Last[GetEnvironment["LIBQCD_INSTALL_PREFIX"]];
If[prefix == None,
   Print["Internal error: no prefix"];
   Abort[]
];

Print["Compiling library"]
dir = FileNameJoin[{"bindings", "Mathematica"}];

code = {
   FileNameJoin[{dir, "bindings.cpp"}]
};

CreateLibrary[code, "libQCD_mathematica",
   "Debug" -> False,
   "Libraries"	-> {"QCD"},
   "Defines" -> "NDEBUG",
   "CleanIntermediate" -> True,
   "Language" -> "C++",
   "CompileOptions" -> "-std=c++11 -Wall -Wextra -Wl,-rpath," <>
   FileNameJoin[{prefix, "lib"}],
   (* "LinkerOptions"-> {"-rpath " <>FileNameJoin[{prefix, "lib"}]}, *)
   "IncludeDirectories" -> {FileNameJoin[{prefix, "include"}]},
   "LibraryDirectories" -> {FileNameJoin[{prefix, "lib"}]}
];

If[$OperatingSystem == "MacOSX",
   Print["Fixing installation paths"];
   Print[RunProcess[{
      "install_name_tool",
      "-change", "libQCD.dylib", FileNameJoin[
         {prefix, "lib", "libQCD.dylib"}
      ],
      FindLibrary["libQCD_mathematica"]
   }]];
];

Print["Installing Mathematica package"];
With[
   {package = "QCD.m"},
   source = FileNameJoin[{dir, package}];
   target = FileNameJoin[{$UserBaseDirectory, "Applications", package}];
   Quiet[DeleteFile[target]];
   CopyFile[source, target];
];
