#include <iostream>
#include <iomanip>
#include <vector>
#include <thread>
#include <future>

#include "QQbar_threshold/load_grid.hpp"
#include "QQbar_threshold/xsection.hpp"

namespace QQt = QQbar_threshold;

struct xs_point{
  double sqrt_s;
  double xs_default;
  double xs_min;
  double xs_max;
};

double xsection(double sqrt_s, double mu){
  static constexpr double mu_width = 350.;
  static constexpr double mt = 171.5;
  static constexpr double width = 1.33;
  return QQt::ttbar_xsection(
      sqrt_s,
      {mu, mu_width},
      {mt, width},
      QQt::N3LO
  );
}

xs_point xsection_scale_variation(double sqrt_s){
  static constexpr double mu_default = 80.;
  static constexpr double mu_min = 50.;
  static constexpr double mu_max = 350.;
  static constexpr double mu_step = 5.;
  xs_point result;
  result.sqrt_s = sqrt_s;
  result.xs_default = xsection(sqrt_s, mu_default);
  result.xs_min = result.xs_default;
  result.xs_max = result.xs_default;
  for(double mu = mu_min; mu < mu_max; mu += mu_step){
    double current_xsection = xsection(sqrt_s, mu);
    if(current_xsection < result.xs_min){
      result.xs_min = current_xsection;
    }
    else if(current_xsection > result.xs_max){
      result.xs_max = current_xsection;
    }
  }
  return result;
}

int main(){
  QQt::load_grid(QQt::grid_directory() + "ttbar_grid.tsv");
  std::vector<std::future<xs_point>> results;
  for(double sqrt_s = 340.; sqrt_s < 349.; sqrt_s += 0.2){
    results.emplace_back(
        std::async(
            std::launch::async,
            xsection_scale_variation, sqrt_s
        )
    );
  }
  std::cout << std::fixed;
  std::cout << "sqrt_s    \tcentral \tmin     \tmax\n";
  for(auto & res: results){
    xs_point current_point = res.get();
    std::cout << current_point.sqrt_s << '\t'
              << current_point.xs_default << '\t'
              << current_point.xs_min << '\t'
              << current_point.xs_max << '\n';
  }
}
