/**
 * @file
 * @brief Shell expansion of path names
 * @author  Andreas Maier <a.maier@tum.de>
 * @version 1.0.1
 *
 * @section LICENSE
 *
 * Copyright 2014, 2015 Andreas Maier
 *
 * This file is part of the QQbar_threshold library.
 *
 * The QQbar_threshold library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 3 of
 * the License, or (at your option) any later version.
 *
 * The QQbar_threshold library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the QQbar_threshold library. If not, see
 * <http://www.gnu.org/licenses/>.
 *
 */
#include <wordexp.h>
#include <string>

namespace word_exp{
  //! Wrapper class for expansion of path names
  class word_exp{
    wordexp_t p;
    char const * path_name;
    int err;
  public:
    //! Construct from path name
    /**
     *@param The unexpanded path name
     */
    word_exp(char const* path);
    //! Get error code in case the expansion failed
    /**
     * See man 3 wordexp for possible error codes
     */
    int get_err() const;
    //! Convert to string containing the expanded path
    explicit operator std::string() const;
    //! Return expanded path
    std::string get_path() const;
    ~word_exp();
  };

  //! Convert to string containing the expanded path
  std::string to_string(word_exp w);
}
