/**
 * @file
 * @brief Implements shell expansion of path names
 *
 * @see word_exp.hpp
 *
 * @author  Andreas Maier <a.maier@tum.de>
 * @version 1.0.1
 *
 * @section LICENSE
 *
 * Copyright 2015 Andreas Maier
 *
 * This file is part of the QQbar_threshold library.
 *
 * The QQbar_threshold library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 3 of
 * the License, or (at your option) any later version.
 *
 * The QQbar_threshold library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the QQbar_threshold library. If not, see
 * <http://www.gnu.org/licenses/>.
 *
 */
#include <word_exp.hpp>
#include <stdexcept>

namespace word_exp{
  word_exp::word_exp(char const* path): path_name(path){
    err = wordexp(path, &p, 0);
  }
  int word_exp::get_err() const{
    return err;
  }
  word_exp::operator std::string() const{
    return get_path();
  }
  std::string word_exp::get_path() const{
    switch(err){
    case WRDE_NOSPACE:
      throw std::bad_alloc();
    case WRDE_BADCHAR:
      throw std::invalid_argument(
	  std::string("Bad char in file name '")
	  + path_name
	  + std::string("'")
      );
    case WRDE_SYNTAX:
      throw std::invalid_argument(
	  std::string("Syntax error in file name '")
	  + path_name
	  + std::string("'")
      );
    }
    if(p.we_wordc != 1){
      throw std::invalid_argument(
	  std::string("Not a single file name: '")
	  + path_name
	  + std::string("'")
      );
    }
    return {p.we_wordv[0]};
  }
  word_exp::~word_exp(){
    wordfree(&p);
  }

  std::string to_string(word_exp w){
    return w.get_path();
  }

}
